@@ -1,5 +1,5 @@
 IBrokersRef <- function() {
-  ref <- system.file("doc/IBrokersRef.pdf",package="IBrokers")
+  ref <- system.file("doc/IBrokersREFCARD.pdf",package="IBrokers")
   if(.Platform$OS.type == "windows") 
     shell.exec(ref)
   else system(paste(shQuote(getOption("pdfviewer")), shQuote(ref)), 
