@@ -4,7 +4,7 @@ twsComboLeg <- function(conId=0,
                         exchange=NULL,
                         openClose=0,
                         shortSaleSlot=0,
-                        designatedLocation=NULL)
+                        designatedLocation="")
 {
   structure(list(conId=conId,
                  ratio=ratio,
